import React, { useState, useEffect } from 'react';
import { BarChart, ComposedChart, RadialBarChart, PieChart, Bar, RadialBar, Pie, Cell, Treemap, XAxis, YAxis, PolarAngleAxis, Legend, CartesianGrid, ResponsiveContainer } from 'recharts';
import { Spinner } from '@/app/components/ui/Spinner';

export const Analytics = props => {
	const { id } = props;
	const [ ready, setReady ] = useState(false);
	const [ selection, setSelection] = useState({});
	const [ recommendations, setRecommendations] = useState({});
	const [ talants, setTalants] = useState({});
	const [ plan, setPlan] = useState({});
	useEffect(() => {
		fetch(`/data/screen3-analytics-${id}.json`, {
			cache: 'no-store'
		}).then((response) => {
			return response.json();
		}).then((data) => {
			setSelection(data.selection);
			setRecommendations(data.recommendations);
			setTalants(data.talants);
			setPlan(data.plan);
			setReady(true);
		}).catch(e => {});
	}, []);
	if (!ready) {
		return <Spinner />;
	}
	const chartHeight = 310;
	const colors = [ "#5BDED8", "#FDCF00", "#3E60FD", "#FF5722", "#7852FB", "#36CA75", "#FF7AED", "#2AB0FC"];
	return (
		<div className="row">
			<div className="col-4">
				<div className="pane">
					{!selection ? (
						<div className='text-center py-3'>
							Данные недоступны
						</div>
					) : (
						<>
							{'title' in selection && (
								<h3 className="mb-4">
									{selection.title}
								</h3>
							)}
							<ResponsiveContainer width="100%" height={chartHeight}>
								<RadialBarChart
									width={130}
									height={130}
									innerRadius="50%"
									data={[ selection ]}
								>
									<PolarAngleAxis type="number" dataKey="value" domain={[0, 100]} angleAxisId={0} tick={false} tickFormatter={null} />
									<RadialBar 
										angleAxisId={0}
										clockWise={true}
										dataKey='value'
										fill="#36CA75"
										background={true}
										label={{ position: 'center', formatter: v => `${v}%`, fill: '#36CA75', fontWeight: 'bold' }}
									/>
								</RadialBarChart>
							</ResponsiveContainer>
						</>
					)}
				</div>
			</div>
			<div className="col-8">
				<div className="pane">
					{!recommendations ? (
						<div className='text-center py-3'>
							Данные недоступны
						</div>
					) : (
						<>
							{'title' in recommendations && (
								<h3 className="mb-4">
									{recommendations.title}
								</h3>
							)}
							<div className="row">
								<div className="col-7">
									<ResponsiveContainer width="100%" height={chartHeight}>
										<Treemap
											data={recommendations.data}
											dataKey="value"
											content={props => {
												const {
													x,
													y,
													width,
													height,
													index,
													children,
													value
												} = props;
												const hasChildren = Array.isArray(children) && children.length > 0;
												return (
													<g>
														<rect
															x={x}
															y={y}
															width={width}
															height={height}
															stroke="#ffffff"
															strokeWidth={hasChildren ? 0 : 2}
															fill={hasChildren ? "#ffffff" : colors[index % colors.length]}
															rx={10}
															ry={10}
														/>
														{!hasChildren && (
															<text
																x={x + width / 2}
																y={y + height / 2 + 7}
																textAnchor="middle"
																fill="#fff"
																fontSize={15}
															>
																{value}%
															</text>
														)}
													</g>
												);
											}}
										/>
									</ResponsiveContainer>
								</div>
								<div className="col-5">
									{recommendations.data.map((dataItem, dataItemIdx) => (
										<div key={dataItemIdx} className="mb-2">
											<span className="d-inline-block align-middle mr-2" style={{ width: '1rem', height: '1rem', borderRadius: '.3125rem', backgroundColor: colors[dataItemIdx % colors.length] }}/>
											{dataItem.name}
										</div>
									))}
								</div>
							</div>
						</>
					)}
				</div>
			</div>
			<div className="col-6">
				<div className="pane">
					{!talants ? (
						<div className='text-center py-3'>
							Данные недоступны
						</div>
					) : (
						<>
							{'title' in talants && (
								<h3 className="mb-4">
									{talants.title}
								</h3>
							)}
							<ResponsiveContainer width="100%" height={chartHeight}>
								<BarChart
									data={talants.data}
									layout="vertical"
									barCategoryGap={3}
									barGap={3}
								>
									<XAxis type="number" orientation="top" />
									<YAxis type="category" dataKey="name" width={150}/>
									<CartesianGrid strokeDasharray="3 3"/>
									<Bar
										dataKey="value"
										fill="#36CA75"
									>
										{talants.data.map((dataItem, dataItemIdx) => (
											<Cell
												key={dataItemIdx}
												fill={colors[(dataItemIdx + 3) % colors.length]}
												radius={[4, 4, 4, 4]}
											/>
										))}
									</Bar>
								</BarChart>
							</ResponsiveContainer>
						</>
					)}
				</div>
			</div>
			<div className="col-6">
				<div className="pane">
					{!talants ? (
						<div className='text-center py-3'>
							Данные недоступны
						</div>
					) : (
						<>
							{'title' in plan && (
								<h3 className="mb-4">
									{plan.title}
								</h3>
							)}
							<ResponsiveContainer width="100%" height={chartHeight}>
								<PieChart width="100%" height={200}>
									<Pie
										dataKey="value"
										nameKey="name"
										data={plan.data}
										innerRadius={45}
										labelLine={false}
										label={props => {
											const { cx, cy, midAngle, innerRadius, outerRadius, value, index } = props;
											const RADIAN = Math.PI / 180;
											const radius = innerRadius + (outerRadius - innerRadius) * 0.5;
											const x = cx + radius * Math.cos(-midAngle * RADIAN);
											const y = cy + radius * Math.sin(-midAngle * RADIAN);
											return (
												<text
													x={x}
													y={y}
													fill="#ffffff"
													textAnchor="middle"
													dominantBaseline="central"
													fontWeight="bold"
													fontSize={15}
												>
													{`${value}%`}
												</text>
											);
										}}
									>
										{plan.data.map((dataItem, dataItemIdx) => (
											<Cell
												key={dataItemIdx}
												fill={colors[dataItemIdx + 3 % colors.length]}
												stroke="#ffffff"
												strokeWidth={4}
											/>
										))}
									</Pie>
									<Legend
										layout="vertical"
										verticalAlign="middle"
										align="right"
									/>
								</PieChart>
							</ResponsiveContainer>
						</>
					)}
				</div>
			</div>
		</div>
	);
}
